==> v0.2.0.tar.gz: rector/packages/BetterReflection/src/SourceLocator/SourceLocatorFactory.php <==
<?php declare(strict_types=1);

namespace Rector\BetterReflection\SourceLocator;

use Rector\BetterReflection\SourceLocator\Ast\Locator;
use Rector\BetterReflection\SourceLocator\Type\AggregateSourceLocator;
use Rector\BetterReflection\SourceLocator\Type\AutoloadSourceLocator;
use Rector\BetterReflection\SourceLocator\Type\DirectoriesSourceLocator;
use Rector\BetterReflection\SourceLocator\Type\MemoizingSourceLocator;
use Rector\BetterReflection\SourceLocator\Type\PhpInternalSourceLocator;
use Rector\BetterReflection\SourceLocator\Type\SingleFileSourceLocator;
use Rector\BetterReflection\SourceLocator\Type\SourceLocator;

final class SourceLocatorFactory
{
    /**
     * @var Locator
     */
    private $locator;

    /**
     * @var StubSourceLocator
     */
    private $stubSourceLocator;

    /**
     * @var SourceLocator[]
     */
    private $commonLocators = [];

    public function __construct(Locator $locator, StubSourceLocator $stubSourceLocator)
    {
        $this->locator = $locator;
        $this->stubSourceLocator = $stubSourceLocator;
    }

    public function create(): SourceLocator
    {
        return $this->wrapInMemoizingSourceLocator($this->createCommonLocators());
    }

    /**
     * @param string[] $source Files or directories
     */
    public function createWithSource(array $source): SourceLocator
    {
        $sourceLocators = [];
        [$directories, $files] = $this->splitToDirectoriesAndFiles($source);

        if (count($directories)) {
            $sourceLocators[] = new DirectoriesSourceLocator($directories, $this->locator);
        }

        foreach ($files as $file) {
            $sourceLocators[] = new SingleFileSourceLocator($file, $this->locator);
        }

        return $this->wrapInMemoizingSourceLocator(
            // order matters to performance
            array_merge($this->createCommonLocators(), $sourceLocators)
        );
    }

    /**
     * @return SourceLocator[]
     */
    private function createCommonLocators(): array
    {
        if ($this->commonLocators) {
            return $this->commonLocators;
        }

        return $this->commonLocators = [
            new AutoloadSourceLocator($this->locator),
            new PhpInternalSourceLocator($this->locator),
            $this->stubSourceLocator,
        ];
    }

    /**
     * @param SourceLocator[] $sourceLocators
     */
    private function wrapInMemoizingSourceLocator(array $sourceLocators): MemoizingSourceLocator
    {
        return new MemoizingSourceLocator(new AggregateSourceLocator($sourceLocators));
    }

    /**
     * @param string[] $source
     * @return string[][]
     */
    private function splitToDirectoriesAndFiles(array $source): array
    {
        $directories = [];
        $files = [];

        foreach ($source as $fileOrDirectory) {
            if (is_dir($fileOrDirectory)) {
                $directories[] = $fileOrDirectory;
            } elseif (file_exists($fileOrDirectory)) {
                $files[] = $fileOrDirectory;
            }
        }

        return [$directories, $files];
    }
}

==> v0.3.3.tar.gz: rector/packages/BetterReflection/src/SourceLocator/SourceLocatorFactory.php <==
<?php declare(strict_types=1);

namespace Rector\BetterReflection\SourceLocator;

use Roave\BetterReflection\SourceLocator\Ast\Locator;
use Roave\BetterReflection\SourceLocator\Type\AggregateSourceLocator;
use Roave\BetterReflection\SourceLocator\Type\AutoloadSourceLocator;
use Roave\BetterReflection\SourceLocator\Type\DirectoriesSourceLocator;
use Roave\BetterReflection\SourceLocator\Type\MemoizingSourceLocator;
use Roave\BetterReflection\SourceLocator\Type\PhpInternalSourceLocator;
use Roave\BetterReflection\SourceLocator\Type\SingleFileSourceLocator;
use Roave\BetterReflection\SourceLocator\Type\SourceLocator;

final class SourceLocatorFactory
{
    /**
     * @var Locator
     */
    private $locator;

    /**
     * @var SourceLocator[]
     */
    private $commonLocators = [];

    public function __construct(Locator $locator)
    {
        $this->locator = $locator;
    }

    public function create(): SourceLocator
    {
        return $this->wrapInMemoizingSourceLocator($this->createCommonLocators());
    }

    /**
     * @param string[] $source Files or directories
     */
    public function createWithSource(array $source): SourceLocator
    {
        $sourceLocators = [];
        [$directories, $files] = $this->splitToDirectoriesAndFiles($source);

        if (count($directories)) {
            $sourceLocators[] = new DirectoriesSourceLocator($directories, $this->locator);
        }

        foreach ($files as $file) {
            $sourceLocators[] = new SingleFileSourceLocator($file, $this->locator);
        }

        return $this->wrapInMemoizingSourceLocator(
            // order matters to performance
            array_merge($this->createCommonLocators(), $sourceLocators)
        );
    }

    /**
     * @return SourceLocator[]
     */
    private function createCommonLocators(): array
    {
        if ($this->commonLocators) {
            return $this->commonLocators;
        }

        return $this->commonLocators = [
            new AutoloadSourceLocator($this->locator),
            new PhpInternalSourceLocator($this->locator),
        ];
    }

    /**
     * @param SourceLocator[] $sourceLocators
     */
    private function wrapInMemoizingSourceLocator(array $sourceLocators): MemoizingSourceLocator
    {
        return new MemoizingSourceLocator(new AggregateSourceLocator($sourceLocators));
    }

    /**
     * @param string[] $source
     * @return string[][]
     */
    private function splitToDirectoriesAndFiles(array $source): array
    {
        $directories = [];
        $files = [];

        foreach ($source as $fileOrDirectory) {
            if (is_dir($fileOrDirectory)) {
                $directories[] = $fileOrDirectory;
            } elseif (file_exists($fileOrDirectory)) {
                $files[] = $fileOrDirectory;
            }
        }

        return [$directories, $files];
    }
}
